//! Standard extensions to [`rcore`].
//!
//! [`rcore`]: crate::rcore

pub mod atomic;
pub mod atomic_object;
pub mod collections;
pub mod inject;
pub mod inlining;
mod local_origin;
pub mod nullable;
pub mod point;
pub mod singular;
pub mod tcast;
pub mod tracing;
pub mod typeless;
mod wrapped_origin;

use std::{error::Error, fmt::Display, sync::Arc};

use crate::func::*;
use crate::mode::*;
use crate::rcore::*;

struct Hex<'a> {
    point: &'a Hash,
}

impl Display for Hex<'_> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        for byte in self.point {
            write!(f, "{byte:02x}")?
        }
        Ok(())
    }
}

pub(crate) fn hex(point: &Hash) -> impl '_ + Display {
    Hex { point }
}

#[cfg(test)]
#[test]
fn hex_test() {
    assert_eq!(
        hex(&(0..32).collect::<Vec<_>>().try_into().unwrap()).to_string(),
        "000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f",
    );
}

impl Display for Address {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}@{}", hex(&self.point), self.index)
    }
}

/// Extension trait for [Serializable]s.
pub trait SerializableExt: Serializable {
    /// Serialize into a [Vec] of bytes.
    fn bytes(&self) -> Vec<u8> {
        let mut vec = Vec::new();
        self.serialize(&mut vec);
        vec
    }
}

impl<S: Serializable> SerializableExt for S {}

/// [`ResolverExt::into_rc`].
pub trait ResolverExt<'a, Ctx: Context<'a>>: Resolver<'a, Ctx> + Sized {
    /// Wrap the resolver into [`Arc`].
    fn into_rc(self) -> Arc<dyn Resolver<'a, Ctx>> {
        Arc::new(self)
    }
}

impl<'a, Ctx: Context<'a>, R: Resolver<'a, Ctx>> ResolverExt<'a, Ctx> for R {}
